@@ -21,10 +21,10 @@ import os
 import stat
 
 from dulwich import __version__ as dulwich_version
-from dulwich.diff_tree import RenameDetector, tree_changes
+from dulwich.diff_tree import RenameDetector, TreeChange as DulwichTreeChange, tree_changes
 from dulwich.index import ConflictedIndexEntry, IndexEntry
 from dulwich.object_store import OverlayObjectStore
-from dulwich.objects import S_IFGITLINK, ZERO_SHA, Blob, Tree
+from dulwich.objects import S_IFGITLINK, ZERO_SHA, Blob, Tree, TreeEntry
 
 from ... import conflicts as _mod_conflicts
 from ... import workingtree as _mod_workingtree
@@ -47,7 +47,7 @@ def changes_between_git_tree_and_working_copy(
     """Determine the changes between a git tree and a working tree with index."""
     to_tree_sha, extras = target.git_snapshot(want_unversioned=want_unversioned)
     store = OverlayObjectStore([source_store, target.store])
-    changes = tree_changes(
+    return tree_changes(
         store,
         from_tree_sha,
         to_tree_sha,
@@ -55,16 +55,7 @@ def changes_between_git_tree_and_working_copy(
         rename_detector=rename_detector,
         want_unchanged=want_unchanged,
         change_type_same=True,
-    )
-    # Convert TreeChange named tuples to plain tuples for dulwich >= 0.22.0
-    if dulwich_version >= (0, 22, 0):
-        changes = (
-            (c.type,
-             (c.old.path, c.old.mode, c.old.sha) if c.old else (None, None, None),
-             (c.new.path, c.new.mode, c.new.sha) if c.new else (None, None, None))
-            for c in changes
-        )
-    return changes, extras
+    ), extras
 
 
 class GitWorkingTreeTests(TestCaseWithTransport):
@@ -167,7 +158,7 @@ class TreeDeltaFromGitChangesTests(TestCase):
             )
         )
         changes = [
-            ("remove", (b"a", stat.S_IFREG | 0o755, b"a" * 40), (b"a", 0, b"a" * 40))
+            DulwichTreeChange(type="remove", old=TreeEntry(b"a", stat.S_IFREG | 0o755, b"a" * 40), new=TreeEntry(b"a", 0, b"a" * 40))
         ]
         self.assertEqual(
             delta,
@@ -208,7 +199,7 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         self.assertEqual(set(expected_extras), set(extras))
 
     def test_empty(self):
-        self.expectDelta([("add", (None, None, None), (b"", stat.S_IFDIR, Tree().id))])
+        self.expectDelta([DulwichTreeChange(type="add", old=None, new=TreeEntry(b"", stat.S_IFDIR, Tree().id))])
 
     def test_added_file(self):
         self.build_tree(["a"])
@@ -218,8 +209,8 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         t.add(b"a", stat.S_IFREG | 0o644, a.id)
         self.expectDelta(
             [
-                ("add", (None, None, None), (b"", stat.S_IFDIR, t.id)),
-                ("add", (None, None, None), (b"a", stat.S_IFREG | 0o644, a.id)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"", stat.S_IFDIR, t.id)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"a", stat.S_IFREG | 0o644, a.id)),
             ]
         )
 
@@ -237,25 +228,17 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         self.store.add_object(newt)
         self.expectDelta(
             [
-                ("modify", (b"", stat.S_IFDIR, oldt.id), (b"", stat.S_IFDIR, newt.id)),
-                ("delete", (b"a", stat.S_IFREG | 0o644, a.id), (None, None, None)),
-                ("add", (None, None, None), (b"b", stat.S_IFREG | 0o644, a.id)),
+                DulwichTreeChange(type="modify", old=TreeEntry(b"", stat.S_IFDIR, oldt.id), new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                DulwichTreeChange(type="delete", old=TreeEntry(b"a", stat.S_IFREG | 0o644, a.id), new=None),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"b", stat.S_IFREG | 0o644, a.id)),
             ],
             tree_id=oldt.id,
         )
         if dulwich_version >= (0, 19, 15):
             self.expectDelta(
                 [
-                    (
-                        "modify",
-                        (b"", stat.S_IFDIR, oldt.id),
-                        (b"", stat.S_IFDIR, newt.id),
-                    ),
-                    (
-                        "rename",
-                        (b"a", stat.S_IFREG | 0o644, a.id),
-                        (b"b", stat.S_IFREG | 0o644, a.id),
-                    ),
+                    DulwichTreeChange(type="modify", old=TreeEntry(b"", stat.S_IFDIR, oldt.id), new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                    DulwichTreeChange(type="rename", old=TreeEntry(b"a", stat.S_IFREG | 0o644, a.id), new=TreeEntry(b"b", stat.S_IFREG | 0o644, a.id)),
                 ],
                 tree_id=oldt.id,
                 rename_detector=RenameDetector(self.store),
@@ -276,8 +259,8 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         self.store.add_object(newt)
         self.expectDelta(
             [
-                ("modify", (b"", stat.S_IFDIR, oldt.id), (b"", stat.S_IFDIR, newt.id)),
-                ("add", (None, None, None), (b"b", stat.S_IFREG | 0o644, a.id)),
+                DulwichTreeChange(type="modify", old=TreeEntry(b"", stat.S_IFDIR, oldt.id), new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"b", stat.S_IFREG | 0o644, a.id)),
             ],
             tree_id=oldt.id,
         )
@@ -285,28 +268,16 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         if dulwich_version >= (0, 19, 15):
             self.expectDelta(
                 [
-                    (
-                        "modify",
-                        (b"", stat.S_IFDIR, oldt.id),
-                        (b"", stat.S_IFDIR, newt.id),
-                    ),
-                    (
-                        "copy",
-                        (b"a", stat.S_IFREG | 0o644, a.id),
-                        (b"b", stat.S_IFREG | 0o644, a.id),
-                    ),
+                    DulwichTreeChange(type="modify", old=TreeEntry(b"", stat.S_IFDIR, oldt.id), new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                    DulwichTreeChange(type="copy", old=TreeEntry(b"a", stat.S_IFREG | 0o644, a.id), new=TreeEntry(b"b", stat.S_IFREG | 0o644, a.id)),
                 ],
                 tree_id=oldt.id,
                 rename_detector=RenameDetector(self.store, find_copies_harder=True),
             )
             self.expectDelta(
                 [
-                    (
-                        "modify",
-                        (b"", stat.S_IFDIR, oldt.id),
-                        (b"", stat.S_IFDIR, newt.id),
-                    ),
-                    ("add", (None, None, None), (b"b", stat.S_IFREG | 0o644, a.id)),
+                    DulwichTreeChange(type="modify", old=TreeEntry(b"", stat.S_IFDIR, oldt.id), new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                    DulwichTreeChange(type="add", old=None, new=TreeEntry(b"b", stat.S_IFREG | 0o644, a.id)),
                 ],
                 tree_id=oldt.id,
                 rename_detector=RenameDetector(self.store, find_copies_harder=False),
@@ -315,14 +286,14 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
     def test_added_unknown_file(self):
         self.build_tree(["a"])
         t = Tree()
-        self.expectDelta([("add", (None, None, None), (b"", stat.S_IFDIR, t.id))])
+        self.expectDelta([DulwichTreeChange(type="add", old=None, new=TreeEntry(b"", stat.S_IFDIR, t.id))])
         a = Blob.from_string(b"contents of a\n")
         t = Tree()
         t.add(b"a", stat.S_IFREG | 0o644, a.id)
         self.expectDelta(
             [
-                ("add", (None, None, None), (b"", stat.S_IFDIR, t.id)),
-                ("add", (None, None, None), (b"a", stat.S_IFREG | 0o644, a.id)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"", stat.S_IFDIR, t.id)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"a", stat.S_IFREG | 0o644, a.id)),
             ],
             [b"a"],
             want_unversioned=True,
@@ -337,8 +308,8 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         t.add(b"a", 0, ZERO_SHA)
         self.expectDelta(
             [
-                ("add", (None, None, None), (b"", stat.S_IFDIR, t.id)),
-                ("add", (None, None, None), (b"a", 0, ZERO_SHA)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"", stat.S_IFDIR, t.id)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"a", 0, ZERO_SHA)),
             ],
             [],
         )
@@ -355,8 +326,8 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         newt.add(b"a", 0, ZERO_SHA)
         self.expectDelta(
             [
-                ("modify", (b"", stat.S_IFDIR, oldt.id), (b"", stat.S_IFDIR, newt.id)),
-                ("modify", (b"a", stat.S_IFREG | 0o644, a.id), (b"a", 0, ZERO_SHA)),
+                DulwichTreeChange(type="modify", old=TreeEntry(b"", stat.S_IFDIR, oldt.id), new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                DulwichTreeChange(type="modify", old=TreeEntry(b"a", stat.S_IFREG | 0o644, a.id), new=TreeEntry(b"a", 0, ZERO_SHA)),
             ]
         )
 
@@ -374,23 +345,15 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         newt.add(b"a", stat.S_IFDIR, newa.id)
         self.expectDelta(
             [
-                ("modify", (b"", stat.S_IFDIR, oldt.id), (b"", stat.S_IFDIR, newt.id)),
-                (
-                    "modify",
-                    (b"a", stat.S_IFREG | 0o644, olda.id),
-                    (b"a", stat.S_IFDIR, newa.id),
-                ),
+                DulwichTreeChange(type="modify", old=TreeEntry(b"", stat.S_IFDIR, oldt.id), new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                DulwichTreeChange(type="modify", old=TreeEntry(b"a", stat.S_IFREG | 0o644, olda.id), new=TreeEntry(b"a", stat.S_IFDIR, newa.id)),
             ],
             want_unversioned=False,
         )
         self.expectDelta(
             [
-                ("modify", (b"", stat.S_IFDIR, oldt.id), (b"", stat.S_IFDIR, newt.id)),
-                (
-                    "modify",
-                    (b"a", stat.S_IFREG | 0o644, olda.id),
-                    (b"a", stat.S_IFDIR, newa.id),
-                ),
+                DulwichTreeChange(type="modify", old=TreeEntry(b"", stat.S_IFDIR, oldt.id), new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                DulwichTreeChange(type="modify", old=TreeEntry(b"a", stat.S_IFREG | 0o644, olda.id), new=TreeEntry(b"a", stat.S_IFDIR, newa.id)),
             ],
             want_unversioned=True,
         )
@@ -402,8 +365,8 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         newt.add(b"a", stat.S_IFREG | 0o644, newa.id)
         self.expectDelta(
             [
-                ("add", (None, None, None), (b"", stat.S_IFDIR, newt.id)),
-                ("add", (None, None, None), (b"a", stat.S_IFREG | 0o644, newa.id)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"", stat.S_IFDIR, newt.id)),
+                DulwichTreeChange(type="add", old=None, new=TreeEntry(b"a", stat.S_IFREG | 0o644, newa.id)),
             ],
             [b"a"],
             want_unversioned=True,
