@@ -16,8 +16,6 @@
 
 """Tests for bzr-fastimport."""
 
-from __future__ import absolute_import
-
 from .... import errors as bzr_errors
 from ....tests import TestLoader
 from ....tests.features import Feature
@@ -25,7 +23,6 @@ from .. import load_fastimport
 
 
 class _FastimportFeature(Feature):
-
     def _probe(self):
         try:
             load_fastimport()
@@ -34,19 +31,23 @@ class _FastimportFeature(Feature):
         return True
 
     def feature_name(self):
-        return 'fastimport'
+        return "fastimport"
 
 
 FastimportFeature = _FastimportFeature()
 
 
 def test_suite():
-    module_names = [__name__ + '.' + x for x in [
-        'test_commands',
-        'test_exporter',
-        'test_branch_mapper',
-        'test_generic_processor',
-        'test_revision_store',
-        ]]
+    module_names = [
+        __name__ + "." + x
+        for x in [
+            "test_commands",
+            "test_exporter",
+            "test_branch_mapper",
+            "test_generic_processor",
+            "test_marks_file",
+            "test_revision_store",
+        ]
+    ]
     loader = TestLoader()
     return loader.loadTestsFromModuleNames(module_names)
