@@ -16,24 +16,31 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-from __future__ import absolute_import
+__all__ = [
+    "TestCaseInTempDir",
+    "TestCaseWithTransport",
+    "TestUtil",
+    "load_tests",
+    "multiply_tests",
+    "quilt_feature",
+]
 
-from .... import tests
 
-from ....tests import (
-    TestUtil,
-    multiply_tests,
-    TestCaseWithTransport,
-    TestCaseInTempDir,
-    )
+from ....tests import TestCaseInTempDir, TestCaseWithTransport, TestUtil, multiply_tests
+from ....tests.features import ExecutableFeature
+
+quilt_feature = ExecutableFeature("quilt")
 
 
 def load_tests(loader, basic_tests, pattern):
     testmod_names = [
-        'test_merge',
-        'test_wrapper',
-        ]
-    basic_tests.addTest(loader.loadTestsFromModuleNames(
-        ["%s.%s" % (__name__, i) for i in testmod_names]))
+        "test_merge",
+        "test_wrapper",
+    ]
+    basic_tests.addTest(
+        loader.loadTestsFromModuleNames(
+            ["{}.{}".format(__name__, i) for i in testmod_names]
+        )
+    )
 
     return basic_tests
