@@ -21,9 +21,12 @@ def load_tests(loader, basic_tests, pattern):
     # unwanted tests and I think that's sufficient.
 
     testmod_names = [
-        'test_auto_upload_hook',
-        'test_upload',
-        ]
-    basic_tests.addTest(loader.loadTestsFromModuleNames(
-        ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+        "test_auto_upload_hook",
+        "test_upload",
+    ]
+    basic_tests.addTest(
+        loader.loadTestsFromModuleNames(
+            ["{}.{}".format(__name__, tmn) for tmn in testmod_names]
+        )
+    )
     return basic_tests
